function [] = tutorial
%
% Function TUTORIAL Demonstration
% Runs a Demo for the following immune tools:
% 1) ABNET (Growing Boolean Competitive Network - journal.doc)
% 2) CLONALG (Clonal Selection Algorithm - gecco00.doc)
% 3) AINET (Artificial Immune Network - ainet.doc)
%
% Other Sources of Reference: RT DCA 01/99 (Artificial Immune Systems Part I - Basic Theory and Applications)
%
% Secondary Functions: 
% Internal Subfunctions: PRESENT, CHOOSE, DRAW_ABNET, PLOTVET, CADEIA
% Auxiliary Subfunctions: ABNET, GA3D, IMALG3D, AINET, ANALYSIS, DENDRO
%
% Copyrigth by Leandro Nunes de Castro
% April/May, 2000
%

% Function chooses the demo
while 1,
   p = present; choose(p);
   disp(sprintf('Pressione qualquer tecla para continuar...'));pause;
end;


% ------------------------- %
%   INTERNAL SUBFUNCTIONS
% ------------------------- %

% Function presents the DEMO TOOLBOX
function p = present,
clc; disp('*-----------------------------------------------*');
disp(sprintf('    ENGENHARIA IMUNOLGICA - DEFESA DE TESE'));
disp(sprintf('    Candidato: Leandro Nunes de Castro'));
disp(sprintf('    Data: 11 de Maio de 2001'));
disp('*-----------------------------------------------*');
disp(sprintf('Demos Disponveis:'));
disp(sprintf('1) SAND'));
disp(sprintf('2) CLONALG'));
disp(sprintf('3) ABNET'));
disp(sprintf('4) aiNet'));
p = input('Escolha o demo desejado (ou CTRL^C para Sair): ');
% End function PRESENT


% Function allows choosing Demos
function choose(p);
switch p,
case 1,
   disp(sprintf('\n** PARTE I - SAND (Simulated ANnealing Approach to Increase Diversity) **'));
   disp(sprintf('Demos disponveis:'));
   disp(sprintf('1) Espao de Formas Binrio de Hamming'));
   disp(sprintf('2) Espao de Formas Euclidiano'));
   disp(sprintf('3) Inicializao de MLP Treinado via Backpropagation'));
   task = input('Escolha o demo desejado (1), (2) or (3): ');
   switch task,
   case 1,
      N = input('Escolha o tamanho N da populao [2,8]: ');
      [ab,vE,E] = sacubic(N);
      figure(3); plot(vE); title('Evoluo da Energia (E)');
      axis([0 length(vE) 0 105]);
   case 2,
      N = input('Escolha o tamanho N da populao: ');
      ab = randn(N,2);
      [ab,vE,vRb,E,Rb] = sareal(ab);
      figure(3); plot(vRb); title('Evoluo da Energia (E)');
      axis([0 length(vRb) 0 105]);
   case 3,
      disp(sprintf('Problema de Paridade 3'));
      nh = 3; minerr = 0.01; maxep = 200; val = 0.1;
      % dados_soja; P = norma(P); T = norma(T);
      load dados_par3; vNep1 = []; vNep2 = [];
      [np,ni] = size(P); [np,no] = size(T);
      disp(sprintf('Nmero mximo de iteraes: %d',maxep));
      disp(sprintf('Nmero de amostras: %d',length(T)));
      disp(sprintf('Critrio de parada: SSE=%.2f',minerr));
      disp(sprintf('Dimenso da rede: [%d,%d,%d]',ni,nh,no));
      disp(sprintf('Pressione qualquer tecla para continuar...'));pause;
      for i=1:3,
         w1 = 2.* val.*rand(ni+1,nh) - val; w1sa = w1;
         w2 = 2.* val.*rand(nh+1,no) - val; w2sa = w2;
         [Nep1,T1] = scgm(P,T,nh,minerr,maxep,w1,w2);
         [w1sa,Tsand] = sareal_init(w1);
         % w1sa = val.*w1sa;
         [Nep2,Tscgm] = scgm(P,T,nh,minerr,maxep,w1sa,w2sa);
         T2 = Tsand + Tscgm;
         vNep1 = [vNep1;Nep1]; vNep2 = [vNep2;Nep2];
      end;
      disp('   Rand  SAND');
      m = [vNep1,vNep2]; disp(m);
      %disp(sprintf('Inicializao aleatria: Nep = %d, T = %.1f',Nep1,T1));
      %disp(sprintf('Inicializao via SAND: Nep = %d, T = %.1f',Nep2,T2));
   otherwise,
      disp('Os valores aceitos so (1), (2) ou (3)');
   end; % End switch task SAD
case 2,
   disp(sprintf('\n** PARTE II - CLONALG (CLONal Selection ALGorithm) **'));
   % figure(1); clf;
   v = cadeia(100,44,0,0,0);
   disp(sprintf('Demos disponveis:'));
   disp(sprintf('1) CHARACTER (APRENDIZAGEM DE MQUINA)'));
   disp(sprintf('2) GA (OTIMIZAO GLOBAL)'));
   disp(sprintf('3) CLONALG (OTIMIZAO MULTIMODAL)'));
   ga_cga = input('Escolha o demo desejado (1), (2) or (3): ');
   switch ga_cga,
   case 1,
      disp(sprintf('** CLONALG - APRENDIZAGEM DE MQUINA**'));
      load num8_12x10; X = v(1,:); P = cadeia(10,120,0,0,0);
      M = imalgchar(P,X);
   case 2,
      disp(sprintf('** Algoritmo Gentico Padro - GA **'));
      figure(1); clf;
      [x,y,fx,vx,vmfit,P] = ga3d(v);
      disp(sprintf('Mximo encontrado [x,y,f(x,y)]: [%.2f,%.2f,%.2f]',x,y,fx));
      figure(2); plot(vx); title('f(x,y) x Mdia'); xlabel('geraes'); ylabel('f(x)');
      hold on; plot(vmfit,'r'); hold off;
   case 3,
      disp(sprintf('** Algoritmo de Seleo Clonal - CLONALG **'));
      figure(3); clf; %v = v(1:50,:);
      [x,y,fx,vfx,vmfit,P,vpm] = imalg3d(v);
      save data x y fx vfx;
      disp(sprintf('Mximo encontrado [x,y,f(x,y)]: [%.2f,%.2f,%.2f]',x,y,fx));
      figure(4); clf; plot(vfx); title('f(x,y) x Mdia'); xlabel('Geraes'); ylabel('f(x)');
      hold on; plot(vmfit,'r'); hold off;
   otherwise,
      disp('Os valores aceitos so (1), (2) ou (3)');
   end; % End Switch ga_cga
case 3,
   disp(sprintf('\n** PARTE III - ABNET (AntiBody NETwork) **'));
   L = 10; aux1 = ones(1,L); aux2 = zeros(1,L);
   ag = [aux1;[0,aux1(2:end)];[0,0,aux1(3:end)];[aux1(1:end-1),0];[aux1(1:end-2),0,0];...
         aux2;[1,aux2(2:end)];[1,1,aux2(3:end)];[aux2(1:end-1),1];[aux2(1:end-2),1,1]];
   disp(sprintf('\nAntgenos (colunas) a Serem Reconhecidos: [%d,%d]',size(ag,1),L)); 
   disp(ag'); ag = ag - 0.1; ag = hardlims(ag);
   eps = input('Escolha o Limiar de Afinidade [0,L]: ');
   [w,win,cwin,D] = abnet(ag,eps);
   figure(2); clf; draw_abnet(w);
case 4,
   disp(sprintf('\n** PARTE IV - aiNet (Artificial Immune NETwork) **'));
   disp(sprintf('Demos disponveis:'));
   disp(sprintf('1) CLASSES LINERAMENTE SEPARVEIS'));
   disp(sprintf('2) 2-SPIRAIS'));
   disp(sprintf('3) CHAINLINK'));
   disp(sprintf('4) DEFINIO DE CENTROS DE RBF - R Auto-Oraganizado'));
   disp(sprintf('5) DEFINIO DE CENTROS DE RBF - R Definido'));
   task = input('Escolha o demo desejado (1), (2), (3), (4) ou (5): ');
   switch task,
   case 1,
      load Ag; Ag = norma(Ag);
      figure(1); plotvet(Ag,10); title('Dados de Entrada');
      ts = input('Escolha o limiar de supresso (sugesto: 0.1): ');
      [M,D] = ainet(Ag,ts,4,10,10,0.2,1);
      [E,Nc,mCe,mC,T] = anal1(M,D);
      disp(sprintf('Nmero de clusters: %d',Nc));
   case 2,
      figure(1); Ag = spir; title('Dados de Entrada');
      ts = input('Escolha o limiar de supresso (sugesto: 0.07): ');
      [M,D] = ainet(Ag,ts,4,10,20,0.1,1);
      [E,Nc,mC,T] = anal2(M,D);
      disp(sprintf('Nmero de clusters: %d',Nc));
   case 3,
      figure(1); Ag = donuts1(500); title('Dados de Entrada');
      ts = input('Escolha o limiar de supresso (sugesto: 0.15): ');
      [M,D] = ainet(Ag,ts,4,10,10,0.1,1);
      [E,Nc,mC,T] = anal3(M,D);
      disp(sprintf('Nmero de clusters: %d',Nc));
   case 4,
      load data_sin1; [np,ni] = size(P);
      [C,D,sig] = ainet_sig2(P,0.5,4,5,5,0.2,1,4,0.01);
      [nm,ni] = size(C);
      disp(sprintf('Compresso de dados: %.2f%%',100-(nm*100)/np));
      disp('Executando a rede RBF...');
      [w,y] = rbf_hybrid(P,T,C,sig);		% Run RBF for all C defined by aiNet
      figure(1); plot(P,T,'+'); hold on; plot(P,y,'r'); 
      plot(C,sin(C),'kp'); title('+: Treinamento, -- : Sada da RBF');
      axis([min(P)-.3 max(P)+.3 min(T)-.1 max(T)+.1]); 
      a = -pi:.1:pi; plot(a,sin(a),'k-.'); hold off;
   case 5,
      load data_sin1; [np,ni] = size(P);
      [C,D] = ainet_rbf(P,0.5,4,5,5,0.2,1,4,0.01); % OK
      [nm,ni] = size(C);
      disp(sprintf('Compresso de dados: %.2f%%',100-(nm*100)/np));
      disp('Executando a rede RBF...');
      [w,y] = rbf_fixed(P,T,C);		% OK
      figure(2); plot(P,T,'+'); hold on; plot(P,y,'r'); 
      plot(C,sin(C),'kp'); title('+: Treinamento, -- : Sada da RBF');
      axis([min(P)-.3 max(P)+.3 min(T)-.1 max(T)+.1]); 
      a = -pi:.1:pi; plot(a,sin(a),'k-.'); hold off;
  otherwise,
      disp('Os valores aceitos so (1) - (5)');
   end; % End Switch task
      
   otherwise,
      disp('Os valores aceitos so (1), (2), (3) ou (4)');
   break;
end;
% End Function CHOOSE


% Function DRAW_ABNET
function draw_abnet(w);
[N,L] = size(w);
r = [0.2.*ones(1,N);1:1:N]';
c = [1.2.*ones(1,L);1:1:L]';
if N > L, y = N; else, y = L; end;
plot(r(:,1),r(:,2),'sk'); hold on;
plot(c(:,1),c(:,2),'or');
axis([0 1.4 0 y+1]);
for i = 1:N,
   for j = 1:L,
      if w(i,j) == 1,
         line([r(i,1),c(j,1)],[r(i,2),c(j,2)]);
      end;
   end;
end;
title('ABNET (0 weights ommitted)');
hold off;
% End Function DRAW_ABNET


% Function CADEIA
function [ab,ag] = cadeia(n1,s1,n2,s2,bip)
if nargin == 2,
   n2 = n1; s2 = s1; bip = 1;
elseif nargin == 4,
   bip = 1;
end;
% Antibody (Ab) chains
ab = 2 .* rand(n1,s1) - 1;
if bip == 1,
   ab = hardlims(ab);
else,
   ab = hardlim(ab);
end;
% Antigen (Ag) chains
ag = 2 .* rand(n2,s2) - 1;
if bip == 1,
   ag = hardlims(ag);
else,
   ag = hardlim(ag);
end;
% End Function CADEIA


% Function Plot Vectors
function [] = plotvet(X,nec)
% nec    -> number of elements per class
clf; hold on;
xlabel('x'); ylabel('y');
plot(X(1:nec,1),X(1:nec,2),'bs');
plot(X(nec+1:2*nec,1),X(nec+1:2*nec,2),'ro');
plot(X(2*nec+1:3*nec,1),X(2*nec+1:3*nec,2),'g+');
plot(X(3*nec+1:4*nec,1),X(3*nec+1:4*nec,2),'kx');
plot(X(4*nec+1:5*nec,1),X(4*nec+1:5*nec,2),'mv');
hold off;
% End Function PLOTVET


% Function normalizes matrix over [0,1]
function [Dn] = norma(D);
% Dn  -> normalized vector over [0,1]
[np,ni] = size(D);
if ni == 1,
   Dn = (D - min(D))./(max(D)-min(D));
else,
   vmaxD = max(D); vminD = min(D);
   for i=1:ni,
      Dn(:,i) = (D(:,i) - vminD(i))./(vmaxD(i)-vminD(i));
   end;
end;
% End Function NORMA